//  Diffusion Client Library for iOS, tvOS and OS X / macOS
//
//  Copyright (c) 2016, 2018 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;
#import <Diffusion/PTDiffusionEnumeration.h>

NS_ASSUME_NONNULL_BEGIN

/**
 Key that may exist in the user info dictionary of an NSError instance
 generated by a request to add a topic.
 
 The corresponding value is a `PTDiffusionTopicAddFailReason` instance.

 @deprecated Since 6.0.6. Will be removed in a future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
extern NSString *const PTDiffusionTopicAddFailReasonErrorKey;

/**
 @brief The reason for failure to add a Topic.
 
 @see PTDiffusionTopicControlFeature

 @deprecated Since 6.0.6. Will be removed in a future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
@interface PTDiffusionTopicAddFailReason : PTDiffusionEnumeration

/**
 Topic already existed with exactly the same specification.

 @since 5.7
 */
+(instancetype)exists;

/**
 Topic already exists at the same path but with a different specification.

 @since 5.7
 */
+(instancetype)existsMismatch;

/**
 The name of the topic was found to be invalid.

 @since 5.7
 */
+(instancetype)invalidName;

/**
 Some aspect of the topic specification failed validation.

 This can include an attempt to add a PTDiffusionTopicType_SingleValue or
 PTDiffusionTopicType_Record topic at a path that is configured for replication.

 @since 5.7
 */
+(instancetype)invalidDetails;

/**
 A user supplied class could either not be found or could not be
 instantiated at the server.

 @since 5.7
 */
+(instancetype)userCodeError;

/**
 A referenced Topic was not found or was not of the correct type.

 @deprecated Since 6.1. This value is no longer used and will be removed in a
 future release.

 @since 5.7
 */
+(instancetype)topicNotFound __deprecated_msg("Will be removed in a future release.");

/**
 The client was denied permission to add a topic of the specified type at
 the given point in the topic hierarchy.

 @since 5.7
 */
+(instancetype)permissionsFailure;

/**
 Initialisation of topic with supplied content failed, possibly because it
 was of the wrong format.

 @since 5.7
 */
+(instancetype)initialiseError;

/**
 An unexpected error occurred whilst attempting to create the Topic. See
 the server log for more details.

 @since 5.7
 */
+(instancetype)unexpectedError;

/**
 When trying to create the topic the cluster was migrating the partition that
 owns the topic. The correct owner could not be identified and the request
 failed. This is a transient failure for the duration of the partition
 migration.
 
 @since 6.0
 */
+(instancetype)clusterRepartition;

/**
 Adding the topic failed because of a license limit.

 @since 6.1
 */
+(instancetype)exceededLicenceLimit;

/**
 Adding the topic failed because a topic owned by a publisher is already bound
 to the parent path.

 @since 6.1
 */
+(instancetype)incompatibleParent;

/**
 Adding a slave topic failed because a topic owned by a publisher is already
 bound to the specified master path.

 @since 6.1
 */
+(instancetype)incompatibleMaster;

/**
 Adding the topic failed because a topic is already bound to the specified path
 but the caller does not have the rights to manage it.

 This can be because the topic is being managed by a component with exclusive
 control over the topic, such as fan-out or a publisher and thus the caller will
 not be able to update or remove the topic.

 If the caller has suitable permissions then it could still subscribe to the
 topic, but the topic's specification may be different from that requested.

 @since 6.1
 */
+(instancetype)existsIncompatible;

@end

NS_ASSUME_NONNULL_END
